--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_Professions

Author: Daphne Pfister <tiiroib02@sneakemail.com>

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local pt = LibStub("LibPeriodicTable-3.1")

local MAJOR, MINOR = "LibFilter-item-Professions", tonumber(("$Revision: 9 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

filter.filtertype = "item"
filter.key = "Professions"

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks

-- Avoid Global Lookups
local _G               = getfenv(0)
local ipairs           = _G.ipairs
local next             = _G.next
local pairs            = _G.pairs
local select           = _G.select
local string           = _G.string
local strjoin          = _G.strjoin
local table            = _G.table
local type             = _G.type
local unpack           = _G.unpack
local GetNumSkillLines = _G.GetNumSkillLines
local GetSkillLineInfo = _G.GetSkillLineInfo
local GetSpellTabInfo  = _G.GetSpellTabInfo
local GetSpellName     = _G.GetSpellName
local BOOKTYPE_SPELL   = _G.BOOKTYPE_SPELL

local strmatch         = string.match
local tinsert          = table.insert
local tsort            = table.sort


-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["Professions"] = true, -- Filter name
			["Type"] = true, -- Filter type
			["Items Craftable by Known Professions"] = true,
			["Items Craftable by Known Primary Professions"] = true,
			["Items Craftable by Unknown Professions"] = true,
			["Items Craftable by Unknown Primary Professions"] = true,
			["Crafted Items"] = true,
			["Items Gathered by Known Professions"] = true,
			["Items Gathered by Known Primary Professions"] = true,
			["Items Gathered by Unknown Professions"] = true,
			["Items Gathered by Unknown Primary Professions"] = true,
			["Gathered Items"] = true,
			["Items Used by Known Professions"] = true,
			["Items Used by Known Primary Professions"] = true,
			["Items Used by Unknown Professions"] = true,
			["Items Used by Unknown Primary Professions"] = true,
			["Trade Materials"] = true,
			["Tools Used by Known Professions"] = true,
			["Tools Used by Known Primary Professions"] = true,
			["Tools Used by Unknown Professions"] = true,
			["Tools Used by Unknown Primary Professions"] = true,
			["Professional Tools"] = true,
			["Recipes for Known Professions"] = true,
			["Recipes for Known Primary Professions"] = true,
			["Recipes for Unknown Professions"] = true,
			["Recipes for Unknown Primary Professions"] = true,
			["Recipes"] = true,
			["Filter by items related to Known / Unknown Professions"] = true,
			["Known Professions"] = true,
			["Match only for professions known to the character"] = true,
			["Unknown Professions"] = true,
			["Match if used only by professions unknown to the character"] = true,
			["Primary and Secondary Professions"] = true,
			["Match for both primary and secondary professions"] = true,
			["Primary Professions Only"] = true,
			["Match only for primary professions"] = true,
		},
	}

	local gGameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gGameLocale] or LL["enUS"]

	for vKey,vValue in pairs(enL) do
		if not L[vKey] then L[vKey] = vValue end
	end
	for vKey,vValue in pairs(L) do
		if vValue == true then L[vKey] = vKey end
	end
end

local TESTTYPE_CRAFTED = "Crafted"
local TESTTYPE_GATHER = "Gather"
local TESTTYPE_MAT = "Mat"
local TESTTYPE_TOOL = "Tool"
local TESTTYPE_RECIPE = "Recipe"

local KNOWN_PROFESSIONS   = 1
local OTHER_PROFESSIONS   = 2
local ALL_PROFESSIONS     = 1
local PRIMARY_PROFESSIONS = 2

local SecondaryProfessions = {
	[GetSpellInfo(2550)] = "Cooking",
	[GetSpellInfo(3273)] = "First Aid",
	[GetSpellInfo(7620)] = "Fishing",
	[GetSpellInfo(921)]  = "Pickpocketing",
}

local TypeLogic = {
	[TESTTYPE_CRAFTED] = {
		Name = {
			[KNOWN_PROFESSIONS] = { -- Even if we can not make yet
				[ALL_PROFESSIONS] = L["Items Craftable by Known Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Craftable by Known Primary Professions"],
			},
			[OTHER_PROFESSIONS] = {
				[ALL_PROFESSIONS] = L["Items Craftable by Unknown Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Craftable by Unknown Primary Professions"],
			},
		},
		Description = L["Crafted Items"],
		Everything = "Tradeskill.Crafted",
		Skills = {
			[GetSpellInfo(2259)] = "Tradeskill.Crafted.Alchemy",
			--[GetSpellInfo(28672)] = "??", -- Transmutation Master
			--[GetSpellInfo(28675)] = "??", -- Potion Master
			--[GetSpellInfo(28677)] = "??", -- Elixir Master
			[GetSpellInfo(9788)] = "Tradeskill.Crafted.Blacksmithing.Armorsmith",
			[GetSpellInfo(2018)] = "Tradeskill.Crafted.Blacksmithing.Basic",
			[GetSpellInfo(17041)]= "Tradeskill.Crafted.Blacksmithing.Weaponsmith.Axesmith",
			[GetSpellInfo(9787)] = "Tradeskill.Crafted.Blacksmithing.Weaponsmith.Basic",
			[GetSpellInfo(17040)]= "Tradeskill.Crafted.Blacksmithing.Weaponsmith.Hammersmith",
			[GetSpellInfo(17039)]= "Tradeskill.Crafted.Blacksmithing.Weaponsmith.Swordsmith",
			[GetSpellInfo(7411)] = "Tradeskill.Crafted.Enchanting",
			[GetSpellInfo(4036)] = "Tradeskill.Crafted.Engineering.Basic",
			[GetSpellInfo(20219)]= "Tradeskill.Crafted.Engineering.Gnomish",
			[GetSpellInfo(20222)]= "Tradeskill.Crafted.Engineering.Goblin",
			[GetSpellInfo(45357)]= "Tradeskill.Crafted.Inscription",
			[GetSpellInfo(25229)]= "Tradeskill.Crafted.Jewelcrafting",
			[GetSpellInfo(2108)] = "Tradeskill.Crafted.Leatherworking.Basic",
			[GetSpellInfo(10656)]= "Tradeskill.Crafted.Leatherworking.Dragonscale",
			[GetSpellInfo(10658)]= "Tradeskill.Crafted.Leatherworking.Elemental",
			[GetSpellInfo(10660)]= "Tradeskill.Crafted.Leatherworking.Tribal",
			[GetSpellInfo(2656)] = "Tradeskill.Crafted.Smelting", -- Mining / Smelting
			[GetSpellInfo(3908)] = "Tradeskill.Crafted.Tailoring.Basic",
			[GetSpellInfo(26798)]= "Tradeskill.Crafted.Tailoring.Mooncloth",
			[GetSpellInfo(26801)]= "Tradeskill.Crafted.Tailoring.Shadoweave",
			[GetSpellInfo(26797)]= "Tradeskill.Crafted.Tailoring.Spellfire",

			[GetSpellInfo(2550)] = "Tradeskill.Crafted.Cooking",
			[GetSpellInfo(3273)] = "Tradeskill.Crafted.First Aid",

			[GetSpellInfo(2842)] = "Tradeskill.Crafted.Poisons",
		},
	},
	[TESTTYPE_GATHER] = {
		Name = {
			[KNOWN_PROFESSIONS] = { -- Even if we can not gather yet
				[ALL_PROFESSIONS] = L["Items Gathered by Known Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Gathered by Known Primary Professions"],
			},
			[OTHER_PROFESSIONS] = {
				[ALL_PROFESSIONS] = L["Items Gathered by Unknown Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Gathered by Unknown Primary Professions"],
			},
		},
		Description = L["Gathered Items"],
		Everything = "Tradeskill.Gather",
		Skills = {
			[GetSpellInfo(13262)]= "Tradeskill.Gather.Disenchant",
			[GetSpellInfo(2383)] = "Tradeskill.Gather.Herbalism", -- Find Herbs
			[GetSpellInfo(31252)]= "Tradeskill.Gather.Prospecting", -- Jewelcrafting / Prospecting
			[GetSpellInfo(51005)]= "Tradeskill.Gather.Milling", -- Inscription / Milling
			[GetSpellInfo(2575)] = "Tradeskill.Gather.Mining", -- Mining / Smelting
			[GetSpellInfo(8613)] = "Tradeskill.Gather.Skinning",

			[GetSpellInfo(7620)] = "Tradeskill.Gather.Fishing",

			[GetSpellInfo(921)]  = "Tradeskill.Gather.Pickpocketing",
		},
	},
	[TESTTYPE_MAT] = {
		Name = {
			[KNOWN_PROFESSIONS] = { -- Even if we can not use yet
				[ALL_PROFESSIONS] = L["Items Used by Known Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Used by Known Primary Professions"],
			},
			[OTHER_PROFESSIONS] = {
				[ALL_PROFESSIONS] = L["Items Used by Unknown Professions"],
				[PRIMARY_PROFESSIONS] = L["Items Used by Unknown Primary Professions"],
			},
		},
		Description = L["Trade Materials"],
		Everything = "Tradeskill.Mat.ByProfession",
		Skills = {
			[GetSpellInfo(2259)] = "Tradeskill.Mat.ByProfession.Alchemy",
			--[GetSpellInfo(28672)] = "??", -- Transmutation Master
			--[GetSpellInfo(28675)] = "??", -- Potion Master
			--[GetSpellInfo(28677)] = "??", -- Elixir Master
			[GetSpellInfo(9788)] = "Tradeskill.Mat.ByProfession.Blacksmithing.Armorsmith",
			[GetSpellInfo(2018)] = "Tradeskill.Mat.ByProfession.Blacksmithing.Basic",
			[GetSpellInfo(17041)]= "Tradeskill.Mat.ByProfession.Blacksmithing.Weaponsmith.Axesmith",
			[GetSpellInfo(9787)] = "Tradeskill.Mat.ByProfession.Blacksmithing.Weaponsmith.Basic",
			[GetSpellInfo(17040)]= "Tradeskill.Mat.ByProfession.Blacksmithing.Weaponsmith.Hammersmith",
			[GetSpellInfo(17039)]= "Tradeskill.Mat.ByProfession.Blacksmithing.Weaponsmith.Swordsmith",
			[GetSpellInfo(7411)] = "Tradeskill.Mat.ByProfession.Enchanting",
			[GetSpellInfo(4036)] = "Tradeskill.Mat.ByProfession.Engineering.Basic",
			[GetSpellInfo(20219)]= "Tradeskill.Mat.ByProfession.Engineering.Gnomish",
			[GetSpellInfo(20222)]= "Tradeskill.Mat.ByProfession.Engineering.Goblin",
			[GetSpellInfo(45357)]= "Tradeskill.Mat.ByProfession.Inscription",
			[GetSpellInfo(25229)]= "Tradeskill.Mat.ByProfession.Jewelcrafting",
			[GetSpellInfo(2108)] = "Tradeskill.Mat.ByProfession.Leatherworking.Basic",
			[GetSpellInfo(10656)]= "Tradeskill.Mat.ByProfession.Leatherworking.Dragonscale",
			[GetSpellInfo(10658)]= "Tradeskill.Mat.ByProfession.Leatherworking.Elemental",
			[GetSpellInfo(10660)]= "Tradeskill.Mat.ByProfession.Leatherworking.Tribal",
			[GetSpellInfo(2656)] = "Tradeskill.Mat.ByProfession.Smelting",
			[GetSpellInfo(3908)] = "Tradeskill.Mat.ByProfession.Tailoring.Basic",
			[GetSpellInfo(26798)]= "Tradeskill.Mat.ByProfession.Tailoring.Mooncloth",
			[GetSpellInfo(26801)]= "Tradeskill.Mat.ByProfession.Tailoring.Shadoweave",
			[GetSpellInfo(26797)]= "Tradeskill.Mat.ByProfession.Tailoring.Spellfire",

			[GetSpellInfo(2550)] = "Tradeskill.Mat.ByProfession.Cooking",
			[GetSpellInfo(3273)] = "Tradeskill.Mat.ByProfession.First Aid",

			[GetSpellInfo(2842)] = "Tradeskill.Mat.ByProfession.Poisons",
		},
	},
	[TESTTYPE_TOOL] = {
		Name = {
			[KNOWN_PROFESSIONS] = { -- Even if we can not use yet
				[ALL_PROFESSIONS] = L["Tools Used by Known Professions"],
				[PRIMARY_PROFESSIONS] = L["Tools Used by Known Primary Professions"],
			},
			[OTHER_PROFESSIONS] = {
				[ALL_PROFESSIONS] = L["Tools Used by Unknown Professions"],
				[PRIMARY_PROFESSIONS] = L["Tools Used by Unknown Primary Professions"],
			},
		},
		Description = L["Professional Tools"],
		Everything = "Tradeskill.Tool",
		Skills = {
			[GetSpellInfo(2259)] = "Tradeskill.Tool.Alchemy",
			[GetSpellInfo(2018)] = "Tradeskill.Tool.Blacksmithing",
			[GetSpellInfo(7411)] = "Tradeskill.Tool.Enchanting",
			[GetSpellInfo(4036)] = "Tradeskill.Tool.Engineering",
			[GetSpellInfo(45357)]= "Tradeskill.Tool.Inscription",
			[GetSpellInfo(25229)]= "Tradeskill.Tool.Jewelcrafting",
			[GetSpellInfo(2575)] = "Tradeskill.Tool.Mining",
			[GetSpellInfo(8613)] = "Tradeskill.Tool.Skinning",

			[GetSpellInfo(2550)] = "Tradeskill.Tool.Cooking",
			[GetSpellInfo(7620)] = "Tradeskill.Tool.Fishing",
		},
	},
	[TESTTYPE_RECIPE] = {
		Name = {
			[KNOWN_PROFESSIONS] = { -- Even if we already known if...
				[ALL_PROFESSIONS] = L["Recipes for Known Professions"],
				[PRIMARY_PROFESSIONS] = L["Recipes for Known Primary Professions"],
			},
			[OTHER_PROFESSIONS] = {
				[ALL_PROFESSIONS] = L["Recipes for Unknown Professions"],
				[PRIMARY_PROFESSIONS] = L["Recipes for Unknown Primary Professions"],
			},
		},
		Description = L["Recipes"],
		Everything = "Tradeskill.Recipe", -- Might be better to use GetItemInfo for recipe
		Skills = {
			[GetSpellInfo(2259)] = "Tradeskill.Recipe.Alchemy",
			[GetSpellInfo(2018)] = "Tradeskill.Recipe.Blacksmithing",
			[GetSpellInfo(7411)] = "Tradeskill.Recipe.Enchanting",
			[GetSpellInfo(4036)] = "Tradeskill.Recipe.Engineering",
			[GetSpellInfo(45357)]= "Tradeskill.Recipe.Inscription",
			[GetSpellInfo(25229)]= "Tradeskill.Recipe.Jewelcrafting",
			[GetSpellInfo(2108)] = "Tradeskill.Recipe.Leatherworking",
			[GetSpellInfo(3908)] = "Tradeskill.Recipe.Tailoring",

			[GetSpellInfo(2550)] = "Tradeskill.Recipe.Cooking",
			[GetSpellInfo(3273)] = "Tradeskill.Recipe.First Aid",
			[GetSpellInfo(7620)] = "Tradeskill.Recipe.Fishing",
		},
	},
}

filter.known_skills = filter.known_skills or {}
local known_skills = filter.known_skills

local function update()
	callbacks:Fire("FilterUpdate")
end

local function UpdateSkills()
	local old_known = {}
	for k in pairs(known_skills) do
		old_known[k] = true
		known_skills[k] = nil
	end     
	for skillIndex = 1, GetNumSkillLines() do
		local skillName, isHeader = GetSkillLineInfo(skillIndex)
		if (isHeader == nil) and skillName then known_skills[skillName] = 1 end
	end
	-- Scan first tab of skills to handle spells such as Herbalism vs Find Herbs etc...
	-- Might need to scan further for rogues for pick pocketing and poisons
	-- What about racial passives when the skills is not actually taken?
	-- Maybe should whitelist exceptions.
	local lastId = select(4, GetSpellTabInfo(1))
	for spellId = 1, lastId do
		local skill = GetSpellName(spellId,BOOKTYPE_SPELL)
		if skill then known_skills[skill] = 1 end
	end

	 for k in pairs(known_skills) do
		 if not old_known[k] then
			 -- We learned a new skill
			 update()
			 return
		 else
			 old_known[k] = nil
		 end
	 end

	 if next(old_known) then
		 -- We forgot an old one
		 update()
		 return
	 end
end

filter.events = filter.events or {}
filter.myFrame = filter.myFrame or CreateFrame("Frame","LibFilter_Professions_Frame")
local events = filter.events
local myFrame = filter.myFrame
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

events.PLAYER_LOGIN = UpdateSkills
events.SKILL_LINES_CHANGED = UpdateSkills
myFrame:RegisterEvent("PLAYER_LOGIN")
myFrame:RegisterEvent("SKILL_LINES_CHANGED")
if next(known_skills) then UpdateSkills() end

filter.author = "Nayala"
filter.name = L["Professions"]
filter.description = L["Filter by items related to Known / Unknown Professions"]

function filter:NewSettings(settings)
	return {
		test = settings and settings.test or TESTTYPE_MAT,
		other = settings and settings.other or false,
		primary = settings and settings.primary or false,
		alt = settings and settings.alt or nil, -- Future
	}
end

function filter:FreeSettings(settings)
end

function filter:DescribeSettings(settings)
	local test = settings and settings.test or TESTTYPE_MAT
	local typelogic = TypeLogic[test] or TypeLogic[TESTTYPE_MAT]

	local known_tag = settings and settings.other and OTHER_PROFESSIONS or KNOWN_PROFESSIONS
	local primary_tag = settings and settings.primary and PRIMARY_PROFESSIONS or ALL_PROFESSIONS
	return typelogic.Name[known_tag][primary_tag]
end
-- /print LibStub("LibFilter-item-Professions"):SettingsOptions({}).args
function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = update

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	if not pSettings then return end

	local test = pSettings.test
	if not test or not TypeLogic[test] then
		test = TESTTYPE_MAT
		pSettings.test = test
	end

	local args = {
		{
			order = 1,
			type = "toggle",
			name = L["Known Professions"],
			desc = L["Match only for professions known to the character"],
			get = function() return not pSettings.other end,
			set = function(v)
							if pSettings.other then
								pSettings.other = false
								vCallback()
							end
						end,
		},
		{
			order = 2,
			type = "toggle",
			name = L["Unknown Professions"],
			desc = L["Match if used only by professions unknown to the character"],
			get = function() return pSettings.other end,
			set = function(v)
							if not pSettings.other then
								pSettings.other = true
								vCallback()
							end
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["Primary and Secondary Professions"],
			desc = L["Match for both primary and secondary professions"],
			get = function() return not pSettings.primary end,
			set = function(v)
							if pSettings.primary then
								pSettings.primary = false
								vCallback()
							end
						end,
		},
		{
			order = 4,
			type = "toggle",
			name = L["Primary Professions Only"],
			desc = L["Match only for primary professions"],
			get = function() return pSettings.primary end,
			set = function(v)
							if not pSettings.primary then
								pSettings.primary = true
								vCallback()
							end
						end,
		},
		{
			order = 5,
			type = "header",
			name = L["Type"],
		},
	}
	local testnames = {}
	for test in pairs(TypeLogic) do
		tinsert(testnames,test)
	end
	tsort(testnames)
	local order = #args
	for _,test in ipairs(testnames) do
		local testlogic = TypeLogic[test]
		order = order + 1
		args[order] = {
			type = "toggle",
			order = order,
			name = testlogic.Description,
			desc = testlogic.Description,
			get = function() return pSettings.test == test end,
			set = function(v)
							if pSettings.test ~= test then
								pSettings.test = test
								vCallback()
							end
						end,
		}
	end
	return { type = "group", desc = "LibFilter_Professions options", args = args }
end

function filter:TestIfMatch(settings,link)
	if not link then return end
	local test = settings and settings.test or TESTTYPE_MAT
	local typelogic = TypeLogic[test] or TypeLogic[TESTTYPE_MAT]
	local primary = settings and settings.primary
	local other = settings and settings.other

	local itemid = link:match("item:(%d+)")
	itemid = tonumber(itemid)

	if not pt:ItemInSet(itemid,typelogic.Everything) then return end

	local match = nil
	for skill,set in pairs(typelogic.Skills) do
		if primary and SecondaryProfessions[skill] then
			-- Skip this skill
		elseif other then
			-- Match if the item is match by any unknown skill
			-- and not known by any of ours
			if pt:ItemInSet(itemid,set) then
				if known_skills[skill] then
					return nil
				else
					match = true
				end
			end
		else
			-- Match if matched by any known skill
			if known_skills[skill] then
				if pt:ItemInSet(itemid,set) then
					return true
				end
			end
		end

	end
	return match
end
