--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
LibFilter_Usable

Author: Daphne Pfister <tiiroib02@sneakemail.com>

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local MAJOR, MINOR = "LibFilter-item-Usable", tonumber(("$Revision: 17 $"):match("(%d+)"))
local filter = LibStub:NewLibrary(MAJOR, MINOR)
if not filter then return end

filter.filtertype = "item"
filter.key = "Usable"

filter.callbacks = filter.callbacks or LibStub("CallbackHandler-1.0"):New(filter)
local callbacks = filter.callbacks

-- Avoid Global Lookups
local _G               = getfenv(0)
local ipairs           = _G.ipairs
local next             = _G.next
local pairs            = _G.pairs
local select           = _G.select
local string           = _G.string
local strjoin          = _G.strjoin
local table            = _G.table
local type             = _G.type
local unpack           = _G.unpack
local GetItemInfo      = _G.GetItemInfo;
local IsConsumableItem = _G.IsConsumableItem;
local IsEquippableItem = _G.IsEquippableItem;
local IsUsableItem     = _G.IsUsableItem;
local UnitLevel        = _G.UnitLevel;

local strmatch         = string.match
local tinsert          = table.insert
local tsort            = table.sort

local TESTTYPE_USE = "Use"
local TESTTYPE_CONSUME = "Consume"
local TESTTYPE_EQUIP = "Equip"
local KNOWN_TESTS = {
	[TESTTYPE_USE] = 1,
	[TESTTYPE_CONSUME] = 1,
	[TESTTYPE_EQUIP] = 1,
}

-- No need for a full blown locale library here
local L
do
	local LL = {
		enUS = {
			["Usable-NAME"] = "Usable", -- Filter name
			["Filter by Usable, Consumable, or Equippable"] = true,
			["Test"] = true,
			["Usable"] = true, -- Filter setting
			["Usable by character"] = true,
			["Consumable"] = true, -- Filter setting
			["Consumable by character"] = true,
			["Equippable"] = true, -- Filter setting
			["Equippable by character"] = true, -- Filter setting
		},
	}

	local gameLocale = GetLocale()
	local enL = LL["enUS"]
	L = LL[gameLocale] or LL["enUS"]

	for k in pairs(enL) do
		if not LL[k] then LL[k] = k end
	end
	for k,v in pairs(L) do
		if v == true then L[k] = k end
	end
end


filter.author = "Nayala"
filter.name = L["Usable-NAME"]
filter.description = L["Filter by Usable, Consumable, or Equippable"]

function filter:NewSettings(settings)
	return {
		test = settings and settings.test or TESTTYPE_USE,
	}
end

function filter:FreeSettings(settings)
end

function filter:DescribeSettings(settings)
	local test = settings and settings.test or TESTTYPE_USE
	if test == TESTTYPE_USE then
		return L["Usable"]
	elseif test == TESTTYPE_CONSUME then
		return L["Consumable"]
	elseif test == TESTTYPE_EQUIP then
		return L["Equippable"]
	else
		return L["Usable"]
	end
end

local function update()
	callbacks:Fire("FilterUpdate")
end

function filter:SettingsOptions(pSettings,pTable,pMethod)
	local vCallback = update

	if pTable then
		if type(pTable) == "string" then
			vCallback = function() pTable[pMethod](pTable,pSettings) end
		else
			vCallback = function() pTable(pSettings) end
		end
	end

	if not pSettings then return end

	local test = pSettings.test
	if not test or not KNOWN_TESTS[test] then
		test = TESTTYPE_USE
		pSettings.test = test
	end

	local args = {
		{
			order = 1,
			type = "header",
			name = L["Test"],
		},
		{
			order = 2,
			type = "toggle",
			name = L["Usable"],
			desc = L["Usable by character"],
			get = function() return pSettings.test == TESTTYPE_USE end,
			set = function(v)
							if pSettings.test ~= TESTTYPE_USE then
								pSettings.test = TESTTYPE_USE
								vCallback()
							end
						end,
		},
		{
			order = 3,
			type = "toggle",
			name = L["Consumable"],
			desc = L["Consumable by character"],
			get = function() return pSettings.test == TESTTYPE_CONSUME end,
			set = function(v)
							if pSettings.test ~= TESTTYPE_CONSUME then
								pSettings.test = TESTTYPE_CONSUME
								vCallback()
							end
						end,
		},
		{
			order = 4,
			type = "toggle",
			name = L["Equippable"],
			desc = L["Equippable by character"],
			get = function() return pSettings.test == TESTTYPE_EQUIP end,
			set = function(v)
							if pSettings.test ~= TESTTYPE_EQUIP then
								pSettings.test = TESTTYPE_EQUIP
								vCallback()
							end
						end,
		},
	}
	return { type = "group", desc = "LibFilter_Usable options", args = args }
end

local tooltip = filter.tooltip or CreateFrame("GameTooltip")
local llines = filter.llines or {}
local rlines = filter.rlines or {}
local DURABILITY_PATTERN = string.match(DURABILITY_TEMPLATE,"(.+) .+/.+") or ""
tooltip:SetOwner(UIParent, "ANCHOR_NONE")
for i=1,30 do
	if not llines[i] then
		local left = tooltip:CreateFontString()
		local right = tooltip:CreateFontString()
		llines[i] = left
		rlines[i] = right
		left:SetFontObject(GameFontNormal)
		right:SetFontObject(GameFontNormal)
		tooltip:AddFontStrings(left,right)
	end
end

local function ClearTooltip()
	tooltip:ClearLines()
	for i=1,30 do
		local left = llines[i]
		local right = rlines[i]
		right:SetText()
		left:SetTextColor(0,0,0)
		right:SetTextColor(0,0,0)
	end
	if not tooltip:IsOwned(UIParent) then
		tooltip:SetOwner(UIParent, "ANCHOR_NONE")
	end
end

local function TooltipSetHyperlink(link)
	ClearTooltip()
	tooltip:SetHyperlink(link)
end

function filter:TestIfMatch(settings,link)
	if not link then return end

	local minLevel = select(5, GetItemInfo(link))
	if minLevel and minLevel > UnitLevel("player") then return end

	local test = settings and settings.test

	if test == TESTTYPE_CONSUME then
		if not IsConsumableItem(link) then return end
	elseif test == TESTTYPE_EQUIP then
		if not IsEquippableItem(link) then return end
	else -- TESTTYPE_USE
		if not IsUsableItem(link) then return end
	end

	-- Check if something else is preventing item from being used.
	local name = select(1, GetItemInfo(link))
	TooltipSetHyperlink(link)
	local lines = tooltip:NumLines()
	local r,g,b
	for i=2,lines do
		local is_red = nil
		r,g,b = llines[i]:GetTextColor()
		if r>.9 and g<.2 and b<.2 then is_red = true end
		r,g,b = rlines[i]:GetTextColor()
		if r>.9 and g<.2 and b<.2 then is_red = true end
		if is_red then
			local text = llines[i]:GetText()
			if not text:find(DURABILITY_PATTERN) then
				return nil
			end
		end
	end
	return true
end
